#ifndef AG102_REGS
#define AG102_REGS
/* Check to see if this file has been included already.  */

/* Different memory size defines used by SDRAM initialization */
#define SDRAM_256MB             0x0100
#define SDRAM_128MB             0x00C0
#define SDRAM_64MB              0x0040
#define SDRAM_32MB              0x0004
#define SDRAM_16MB              0x0000
#define ORIG_RAM_BASE           0x80000000

/* DDR2 Controller */
#define DDR2C_BASE              0x90500000       /* Define base for all ddr2 controller registers */
#define DDR2C_CCR_OFFSET        0x00             /* Controller Configuration Register */
#define DDR2C_DCR_OFFSET        0x04             /* DRAM Configuration Register */
#define DDR2C_IOCR_OFFSET       0x08             /* I/O Configuration Register */
#define DDR2C_CSR_OFFSET        0x0C             /* Controller Status Register */
#define DDR2C_DRR_OFFSET        0x10             /* DRAM Refresh Register */
#define DDR2C_TPR_OFFSET(N)     (0x14 + 4 * (N)) /* SDRAM Timing Parameters Registers 0-2 */
#define DDR2C_GDLLCR_OFFSET     0x20             /* Global DLL Control Register */
#define DDR2C_DLLCR_OFFSET(N)   (0x24 + 4 * (N)) /* DLL Control Registers 0-9 */
#define DDR2C_RSLR_OFFSET(N)    (0x4C + 4 * (N)) /* Rank System Latency Registers 0-3 */
#define DDR2C_RDGR_OFFSET(N)    (0x5C + 4 * (N)) /* Rank DQS Gating Registers 0-3 */
#define DDR2C_DQTR_OFFSET(N)    (0x6C + 4 * (N)) /* DQ Timing Registers 0-8 */
#define DDR2C_DQSTR_OFFSET      0x90             /* DQS Timing Register */
#define DDR2C_DQSBTR_OFFSET     0x94             /* DQS_b Timing Register */
#define DDR2C_ODTCR_OFFSET      0x98             /* ODT Configuration Register */
#define DDR2C_DTR_OFFSET(N)     (0x9C + 4 * (N)) /* Data Training Registers 0-1 */
#define DDR2C_DTAR_OFFSET       0xA4             /* Data Training Address Register */
#define DDR2C_MR_OFFSET         0x1F0            /* Mode Register */

#define TIMER_BASE              0x94900000  /* Define base for all timer registers */
#define TIMER1_CNT_OFFSET       0x00        /* Offset for timer1 counter register */
#define TIMER1_LOAD_OFFSET      0x04        /* Offset for timer1 load register */
#define TIMER1_MATCH1_OFFSET    0x08        /* Offset for timer1 counter */
#define TIMER1_MATCH2_OFFSET    0x0c        /* Offset for timer1 counter */
#define TIMER2_CNT_OFFSET       0x10        /* Offset for timer2 counter register */
#define TIMER2_LOAD_OFFSET      0x14        /* Offset for timer2 load register */
#define TIMER2_MATCH1_OFFSET    0x18        /* Offset for timer2 counter */
#define TIMER2_MATCH2_OFFSET    0x1c        /* Offset for timer2 counter */
#define TIMER3_CNT_OFFSET       0x20        /* Offset for timer3 counter register */
#define TIMER3_LOAD_OFFSET      0x24        /* Offset for timer3 load register */
#define TIMER3_MATCH1_OFFSET    0x28        /* Offset for timer3 counter */
#define TIMER3_MATCH2_OFFSET    0x2c        /* Offset for timer3 counter */
#define TIMER_CNTRL_OFFSET      0x30        /* Offset for timer control register */
#define TIMER_INTS_OFFSET       0x34        /* Offset for timer interrupt status register */
#define TIMER_INTM_OFFSET       0x38        /* Offset for timer interrupt mask register */

/* interrupt sources */
#define IC_SWI      (1 << IRQ_SWI)
#define IC_GPIO     (1 << IRQ_GPIO)

#define IC_TIMER1   (1 << IRQ_TIMER1)
#define IC_TIMER2   (1 << IRQ_TIMER2)
#define IC_TIMER3   (1 << IRQ_TIMER3)

/* Timer 1 ~ 3, ATC010 */
#define TM1_CNTR  ((0x000 + TIMER_BASE))
#define TM1_LOAD  ((0x004 + TIMER_BASE))
#define TM1_MTCH1 ((0x008 + TIMER_BASE))
#define TM1_MTCH2 ((0x00c + TIMER_BASE))

#define TM2_CNTR  ((0x010 + TIMER_BASE))
#define TM2_LOAD  ((0x014 + TIMER_BASE))
#define TM2_MTCH1 ((0x018 + TIMER_BASE))
#define TM2_MTCH2 ((0x01c + TIMER_BASE))

#define TM3_CNTR  ((0x020 + TIMER_BASE))
#define TM3_LOAD  ((0x024 + TIMER_BASE))
#define TM3_MTCH1 ((0x028 + TIMER_BASE))
#define TM3_MTCH2 ((0x02c + TIMER_BASE))

#define TM_CR     ((0x030 + TIMER_BASE))
#define TM_STA    ((0x034 + TIMER_BASE))
#define TM_MSK    ((0x038 + TIMER_BASE))

/* TmCR */
#define TM_CR_TM1ENA (0x1 << 0)  //1/enable
#define TM_CR_TM1CLK (0x1 << 1)  //1/EXT1CLK, 0/PCLK
#define TM_CR_TM1OEN (0x1 << 2)  //1/enable overflow interrupt
#define TM_CR_TM2ENA (0x1 << 3)  //1/enable
#define TM_CR_TM2CLK (0x1 << 4)  //1/EXT1CLK, 0/PCLK
#define TM_CR_TM2OEN (0x1 << 5)  //1/enable overflow interrupt
#define TM_CR_TM3ENA (0x1 << 6)  //1/enable
#define TM_CR_TM3CLK (0x1 << 7)  //1/EXT1CLK, 0/PCLK
#define TM_CR_TM3OEN (0x1 << 8)  //1/enable overflow interrupt
#define TM_CR_TM1UP  (0x1 << 9)  //1/up count
#define TM_CR_TM2UP  (0x1 << 10) //1/up count
#define TM_CR_TM3UP  (0x1 << 11) //1/up count

/* IntrState */
#define TM_STA_TM1M1 (0x1 << 0)
#define TM_STA_TM1M2 (0x1 << 1)
#define TM_STA_TM1OV (0x1 << 2)
#define TM_STA_TM2M1 (0x1 << 3)
#define TM_STA_TM2M2 (0x1 << 4)
#define TM_STA_TM2OV (0x1 << 5)
#define TM_STA_TM3M1 (0x1 << 6)
#define TM_STA_TM3M2 (0x1 << 7)
#define TM_STA_TM3OV (0x1 << 8)

/* IntrMask */
#define TM_MSK_TM1M1 TM_STA_TM1M1
#define TM_MSK_TM1M2 TM_STA_TM1M2
#define TM_MSK_TM1OV TM_STA_TM1OV
#define TM_MSK_TM2M1 TM_STA_TM2M1
#define TM_MSK_TM2M2 TM_STA_TM2M2
#define TM_MSK_TM2OV TM_STA_TM2OV
#define TM_MSK_TM3M1 TM_STA_TM3M1
#define TM_MSK_TM3M2 TM_STA_TM3M2
#define TM_MSK_TM3OV TM_STA_TM3OV

#define INTC_TIMER1_MASK        0x00080000  /* Mask to enable timer interrupt */
#define INTC_TIMER1_CLEAR_VALUE 0x00080000  /* Value to clear pending timer1 interrupt */
#define INTC_TIMER_CLEAR_VALUE  0x0008c000  /* Value to clear all pending timer interrupt */
#define INTC_TIMER1_TRIGGER     0x00080000  /* bit 19  = 1: Edge trigger */
#define INTC_TIMER1_LEVEL       0x00000000  /* bit 19  = 0: Falling edge Trigger */

#define TIMER1_INTM_INIT        0x00000003  /* Mask out timer1 match interrupts */
#define TIMER_CNTRL_DISABLE     0x00000000  /* Value to disable timer */
#define TIMER1_CTRL_INIT        0x00000005  /* Bit 2   = 1, enalbe timer1 interrupt */
											/* Bit 1   = 1, clock source, PCLK */
											/* Bit 0   = 1, enable timer1 */
#define TIMER1_INTS_CLEAR_MASK  0xfffffff8  /* Clear mask of timer status register */

/************************************
 *  TARGET INTERRUPT CONSTANTS      *
 ************************************/
#define INTC_BASE               0x90F00000  /* Interrupt Controller base address */
#define IRQREV_OFFSET           0x0         /* INTC revision register */
#define IRQOWNER_OFFSET         0x4         /* CPU owner DC regsiter */
#define IRQCPUID0_OFFSET        0x8         /* CPU ID configuration register for interrupt input 0-15 */
#define IRQCPUID1_OFFSET        0xc         /* CPU ID configuration register for interrupt intpu 16-31 */
#define IRQTMR_OFFSET           0x20        /* Interrupt trigger mode register */
#define IRQTLR_OFFSET           0x24        /* Interrupt trigger level register */
#define FIQTMR_OFFSET           IRQTMR_OFFSET
#define FIQTLR_OFFSET           IRQTLR_OFFSET
#define IRQSRC_OFFSET           0x28        /* Interrupt source register */
#define IRQIPI_TRIG_OFFSET      0x40        /* IPI trigger register */
#define IRQIPI_STAT_OFFSET      0x44        /* IPI status / clear register */
#define IRQIPI_PRIO_OFFSET      0x48        /* IPI priority level register */
#define IRQIPIGLB_STAT_OFFSET   0x4c        /* IPI global status / clear register */
#define IRQIPIGLB_PRIO_OFFSET   0x50        /* IPI global priority register */
#define IRQER_OFFSET            0x80        /* Interrupt enable register */
#define IRQICR_OFFSET           0x84        /* Interrupt status / clear register */
#define FIQER_OFFSET            IRQER_OFFSET
#define FIQICR_OFFSET           IRQICR_OFFSET
#define IRQICR_IVIC_OFFSET(N)   (0x88 + 4*(N))/* IVIC HW0 ~ HW5 interrupt status / clear register */
#define IRQPRI_OFFSET(N)        (0xa0 + 4 *(N))/* Interrupt priority level configuration for interrupt input 0-7 */

#define INT_ER_INIT             0x00000000  /* values used to disable all interrupts */
#define INT_CLEAR_VALUE         0xFFFFFFFF  /* values used to clear all pending interrupts */

/************************************
 *  TARGET DECODER CONSTANTS        *
 ************************************/

#define AHBC_BASE               0x90C00000  /* AHB Controller base address */
#define DEVICE4_OFFSET          0x10        /* Device 4 Control Register offset */
#define DEVICE6_OFFSET          0x18        /* Device 6 Control Register offset */
#define INTC_OFFSET             0x88        /* Interrupt Control Register offset */

#define AHBC_DEV6_INIT          0x10080000  /* AHBC Device 6 init value */
#define AHBC_REMAP_BIT          0x00000001  /* AHBC remap bit for Interrupt Control Register */

/************************************
 * PCU - AG102 Core APB             *
 ************************************/
#define PCU_BASE                0x94800000  /* Device base address */

#define PCU_RR_OFFSET           0x00        /* PCU revision register */
#define PCU_SPIR_OFFSET         0x04        /* Scratch pad info register */
#define PCU_SOCID_OFFSET        0x10        /* SoC ID */
#define PCU_AHBCFG_OFFSET       0x14        /* SoC AHB configuration register */
#define PCU_APBCFG_OFFSET       0x18        /* SoC APB configuration register */
#define PCU_DCSRC0_OFFSET       0x20        /* Driving capability and slew rate control register 0 */
#define PCU_DCSRC1_OFFSET       0x24        /* Driving capability and slew rate control register 1 */
#define PCU_DCSRC2_OFFSET       0x28        /* Driving capability and slew rate control register 2 */
#define PCU_MFPS0_OFFSET        0x30        /* Multi-Function port setting register 0 */
#define PCU_MFPS1_OFFSET        0x34        /* Multi-Function port setting register 1 */
#define PCU_DMASEL_OFFSET       0x38        /* DMA engine selection */
#define PCU_OSCC_OFFSET         0x40        /* OSC control register */
#define PCU_PWMCD_OFFSET        0x44        /* PMW clock divider */
#define PCU_SOC_MISC_OFFSET     0x48        /* SoC misc */
#define PCU_BSMC_OFFSET         0x80        /* BSM control register */
#define PCU_BSMS_OFFSET         0x84        /* BSM status register */
#define PCU_WESENS_OFFSET       0x88        /* Wakeup event sensitivity register */
#define PCU_WESTAT_OFFSET       0x8c        /* Wakeup event status register */
#define PCU_RT_OFFSET           0x90        /* Reset timing register */
#define PCU_ISR_OFFSET          0x94        /* PCU interrupt status register */
#define PCU_PMSP_MEM0_OFFSET(N) (0x400 + 4 * (N)) /* Power Manager Scratch Pad Memory 0 */

/* PCSn registers (n = 1 ~ 9) */
#define PCU_PCSC_OFFSET(N)      (0xa0 + 0x20 * ((N) - 1)) /* PCSn configuration register (clock scaling slot) */
#define PCU_PCSP_OFFSET(N)      (0xa4 + 0x20 * ((N) - 1)) /* PCSn parameter register (clock scaling slot) */
#define PCU_PCSSTAT1_OFFSET(N)  (0xa8 + 0x20 * ((N) - 1)) /* PCSn status register 1 (clock scaling slot) */
#define PCU_PCSSTAT2_OFFSET(N)  (0xac + 0x20 * ((N) - 1)) /* PCSn status register 2 (clock scaling slot) */
#define PCU_PSC1PDD_OFFSET(N)   (0xb0 + 0x20 * ((N) - 1)) /* PCSn PDD register (clock scaling slot) */

/************************************
 *  GPIO controller                 *
 ************************************/
#define GPIO_BASE               0x94C00000  /* GPIO base address */
#define GPIO_DOUT_OFFSET        0x0
#define GPIO_DIN_OFFSET         0x4
#define GPIO_DIR_OFFSET         0x8
#define GPIO_DSET_OFFSET        0x10
#define GPIO_DCLEAR_OFFSET      0x14
#define GPIO_PE_OFFSET          0x18
#define GPIO_PT_OFFSET          0x1C
#define GPIO_IE_OFFSET          0x20
#define GPIO_IS_OFFSET          0x24

#define GPIO_IT_OFFSET          0x34
#define GPIO_IC_OFFSET          0x30
#define GPIO_IB_OFFSET          0x38
#define GPIO_IR_OFFSET          0x3c
#define GPIO_BE_OFFSET          0x40        /* BounceEnable */
#define GPIO_BPS_OFFSET         0x44        /* BouncePreScale */

#define GPIO_USED_MASK          0x0000000A

/******************************************************************
 * UART register mapping
 *****************************************************************/
#define UART_BASE               0x94200000 //0x99600000
#define UART_RBR_OFFSET         0x00    /* Receiver Buffer Register for Read */
#define UART_THR_OFFSET         0x00    /* Transmitter Holding Register for Write */
#define UART_IER_OFFSET         0x04    /* Interrupt Enable Register */
/* Baud-Rate Divisor Latch (DL, Offset: 0x00, 0x04 when DLAB = 1) */
#define UART_IIR_OFFSET         0x08    /* Interrupt Identification Register */
#define UART_FCR_OFFSET         0x08    /* FIFO Control Register */
/* Prescaler Register (PSR, Offset: 0x08 when DLAB =1) */
#define UART_LCR_OFFSET         0x0C    /* Line Control Register */
#define UART_MCR_OFFSET         0x10    /* Modem Control Register */
#define UART_LSR_OFFSET         0x14    /* Line Status Register for Read */
#define UART_TST_OFFSET         0x14    /* Testing Register for Write */
#define UART_MSR_OFFSET         0x18    /* Modem Status Register */
#define UART_SPR_OFFSET         0x1C    /* Scratch Pad Register */
#define UART_MDR_OFFSET         0x20    /* Mode Definition Register */
#define UART_ACR_OFFSET         0x24    /* Auxiliary Control Register */
#define UART_TXLENL_OFFSET      0x28    /* Transmit Frame-Length Register (Low) */
#define UART_TXLENH_OFFSET      0x2C    /* Transmit Frame-Length Register (High)*/
#define UART_MRXLENL_OFFSET     0x30    /* Maximum Receiver Frame-Length (Low) */
#define UART_MRXLENH_OFFSET     0x34    /* Maximum Receiver Frame-Length (High)*/
#define UART_PLR_OFFSET         0x38    /* Preamble Length Register */
#define UART_FMIIR_PIO_OFFSET   0x3C    /* Interrupt Identification Register in FIR Mode (PIO mode) */
#define UART_FMIIR_DMA_OFFSET   0x3C    /* Interrupt Identification Register in FIR Mode (DMA mode) */
#define UART_FMIIER_PIO_OFFSET  0x40    /* Interrupt Identification Enable Register in FIR Mode (PIO mode) */
#define UART_FMIIER_DMA_OFFSET  0x40    /* Interrupt Identification Enable Register in FIR Mode (DMA mode) */
#define UART_STFF_STS_OFFSET    0x44    /* Status FIFO Line Status Register */
#define UART_STFF_RXLENL_OFFSET 0x48    /* Status FIFO Received Frame-Length Register Low */
#define UART_STFF_RXLENH_OFFSET 0x4C    /* Status FIFO Received Frame-Length Register High */
#define UART_FMLSR_OFFSET       0x50    /* Link Status Register in FIR Mode */
#define UART_FMLSIER_OFFSET     0x54    /* Link Status Interrupt Enable Register in FIR Mode */
#define UART_RSR_OFFSET         0x58    /* Resume Register */
#define UART_RXFF_CNTR_OFFSET   0x5C    /* RX FIFO Count Register */
#define UART_LSTFMLENL_OFFSET   0x60    /* Last Frame Length Register Low */
#define UART_LSTFMLENH_OFFSET   0x64    /* Last Frame Length Register High */
#define UART_FEATURE_OFFSET     0x68    /* Feature Register */
#define UART_REVD1_OFFSET       0x6C    /* Revision Register Digit 1 */
#define UART_REVD2_OFFSET       0x70    /* Revision Register Digit 2 */
#define UART_REVD3_OFFSET       0x74    /* Revision Register Digit 3 */
 

#define inw(reg)        (*((volatile unsigned int *) (reg)))
#define outw(reg, data) ((*((volatile unsigned int *)(reg)))=(unsigned int)(data))

#ifndef NDS32_INTR_DEF
/*
 * NDS32_REG_SET_BITS(addr, mask)
 * Do set bits to 1 at specified location
 * Operation: *addr = (*addr | mask)
 */
#define NDS32_SET_BITS(addr, mask) outw(addr, inw(addr) | mask)

/*
 * NDS32_REG_WAIT4_BIT_ON(addr, mask)
 * Read the contents at addr and use mask to
 * mask off unused bits then wait until all of
 * the remaining bits are on
 *
 * Operation: while(!(*addr & mask));
 */
#define NDS32_REG_WAIT4_BIT_ON(addr, mask) while(0 == (inw(addr) & mask))

/*
 * NDS32_REG_WAIT4_BIT_OFF(addr, mask)
 * Read the contents at addr and use mask to
 * mask off unused bits then wait until all of
 * the remaining bits are off
 *
 * Operation: while((*addr & mask));
 */
#define NDS32_REG_WAIT4_BIT_OFF(addr, mask) while(inw(addr) & mask)

/*
 * NDS32_REG_WRITE(addr, data)
 * Write data to specified location at addr
 *
 * Operation: *addr = data
 * */
#define NDS32_REG_WRITE(addr, data) outw(addr, data)
#define NDS32_REG_READ(addr)        inw(addr)

/*
 * NDS32_FMEMCPY_BYTE(dst, src, size)
 * Do forward (low address to high address) memory copy in byte
 * */
#define NDS32_FMEMCPY_BYTE(dst, src, size) \
	__asm__ __volatile__( \
		"sethi   $r3, hi20(%2)       \n\t" \
		"ori     $r3, $r3, lo12(%2)  \n\t" \
		"sethi   $r1, hi20(%0)       \n\t" \
		"ori     $r1, $r1, lo12(%0)  \n\t" \
		"sethi   $r2, hi20(%1)       \n\t" \
		"ori     $r2, $r2, lo12(%1)  \n\t" \
		"1:\n\t" \
		"lbi.bi  $r0, [$r2], 1       \n\t" \
		"sbi.bi  $r0, [$r1], 1       \n\t" \
		"addi    $r3, $r3, -1        \n\t" \
		"bnez    $r3, 1b             \n\t" \
		::"i"(dst),"i"(src),"i"(size):"$r0", "$r1","$r2","$r3")

#define NDS32_FMEMCPY(dst, src, size) \
	__asm__ __volatile__( \
		"sethi   $r3, hi20(%2)       \n\t" \
		"ori     $r3, $r3, lo12(%2)  \n\t" \
		"sethi   $r1, hi20(%0)       \n\t" \
		"ori     $r1, $r1, lo12(%0)  \n\t" \
		"sethi   $r2, hi20(%1)       \n\t" \
		"ori     $r2, $r2, lo12(%1)  \n\t" \
		"1:\n\t" \
		"lwi.bi  $r0, [$r2], 4       \n\t" \
		"swi.bi  $r0, [$r1], 4       \n\t" \
		"addi    $r3, $r3, -4        \n\t" \
		"bgtz    $r3, 1b             \n\t" \
		::"i"(dst),"i"(src),"i"(size):"$r0", "$r1","$r2","$r3")

#endif

/******************************************************************
 * AHB/APB device register mapping
 *****************************************************************/
//irq number
#define IRQ_CFC_CD              0
#define IRQ_CFC_DMA             1
#define IRQ_SSP1                2
#define IRQ_I2C                 3
#define IRQ_SDC                 5
#define IRQ_SSP2                6
#define IRQ_STUART              7
#define IRQ_PMU                 8
#define IRQ_SWI                 9
#define IRQ_FTUART              10
#define IRQ_BTUART              11
#define IRQ_EXTINT5             12
#define IRQ_GPIO                13
#define IRQ_TIMER2              14
#define IRQ_TIMER3              15
#define IRQ_WDT                 16
#define IRQ_RTCALARM            17
#define IRQ_RTCSECOND           18
#define IRQ_TIMER1              19
#define IRQ_CPE_AHB_DMA         21
#define IRQ_CPE_APB_DMA         24
#define IRQ_MAC                 25
#define IRQ_USBDEV              26   // USB device
#define IRQ_EXTINT4             27
#define IRQ_EXTINT0             28
#define IRQ_EXTINT1             29
#define IRQ_EXTINT2             30
#define IRQ_EXTINT3             31

#endif  /* AG102_REGS */
