##==============================================================================
##
##  crt0.S
##
##  nds32 startup code
##
##==============================================================================
##
## Copyright (c) 1995, 1996, 1997, 1998 Cygnus Solutions
##
## The authors hereby grant permission to use, copy, modify, distribute,
## and license this software and its documentation for any purpose, provided
## that existing copyright notices are retained in all copies and that this
## notice is included verbatim in any distributions. No written agreement,
## license, or royalty fee is required for any of the authorized uses.
## Modifications to this software may be copyrighted by their authors
## and need not follow the licensing terms described here, provided that
## the new terms are clearly indicated on the first page of each file where
## they apply.
##

#include "config.h"
#include "interrupt.h"
##------------------------------------------------------------------------------

	.section .nds32_init, "ax"
!========================================================================
! Vector table
!========================================================================
	.align 3
exception_vector:
	j _start                    !  (0) Trap Reset
	j OS_Trap_TLB_Fill          !  (1) Trap TLB fill
	j OS_Trap_PTE_Not_Present   !  (2) Trap PTE not present
	j OS_Trap_TLB_Misc          !  (3) Trap TLB misc
	j OS_Trap_TLB_VLPT_Miss     !  (4) Trap TLB VLPT miss
	j OS_Trap_Machine_Error     !  (5) Trap Machine error
	j OS_Trap_Debug_Related     !  (6) Trap Debug related
	j OS_Trap_General_Exception !  (7) Trap General exception
	j OS_Trap_Syscall           !  (8) Syscall
	j OS_Trap_Interrupt_HW0     !  (9) Interrupt HW0 (for IVB.IVIC_VER = 0)
	j OS_Trap_Interrupt_HW1     ! (10) Interrupt HW1 (for IVB.IVIC_VER = 0)
	j OS_Trap_Interrupt_UNDEF   ! (11) Interrupt HW2
	j OS_Trap_Interrupt_UNDEF   ! (12) Interrupt HW3
	j OS_Trap_Interrupt_UNDEF   ! (13) Interrupt HW4
	j OS_Trap_Interrupt_UNDEF   ! (14) Interrupt HW5
	j OS_Trap_SWI               ! (15) Interrupt HW6 (for IVB.IVIC_VER = 0)
	j OS_Trap_Interrupt_UNDEF   ! (16) Interrupt HW7
	j OS_Trap_Interrupt_UNDEF   ! (17) Interrupt HW8
	j OS_Trap_Interrupt_HW9     ! (18) Interrupt HW9
	j OS_Trap_Interrupt_UNDEF   ! (19) Interrupt HW10
	j OS_Trap_Interrupt_UNDEF   ! (20) Interrupt HW11
	j OS_Trap_Interrupt_UNDEF   ! (21) Interrupt HW12
	j OS_Trap_Interrupt_HW13    ! (22) Interrupt HW13
	j OS_Trap_Interrupt_UNDEF   ! (23) Interrupt HW14
	j OS_Trap_Interrupt_UNDEF   ! (24) Interrupt HW15
	j OS_Trap_Interrupt_UNDEF   ! (25) Interrupt HW16
	j OS_Trap_Interrupt_UNDEF   ! (26) Interrupt HW17
	j OS_Trap_Interrupt_UNDEF   ! (27) Interrupt HW18
	j OS_Trap_Interrupt_HW19    ! (28) Interrupt HW19
	j OS_Trap_Interrupt_UNDEF   ! (29) Interrupt HW20
	j OS_Trap_Interrupt_UNDEF   ! (30) Interrupt HW21
	j OS_Trap_Interrupt_UNDEF   ! (31) Interrupt HW22
	j OS_Trap_Interrupt_UNDEF   ! (32) Interrupt HW23
	j OS_Trap_Interrupt_UNDEF   ! (33) Interrupt HW24
	j OS_Trap_Interrupt_UNDEF   ! (34) Interrupt HW25
	j OS_Trap_Interrupt_UNDEF   ! (35) Interrupt HW26
	j OS_Trap_Interrupt_UNDEF   ! (36) Interrupt HW27
	j OS_Trap_Interrupt_UNDEF   ! (37) Interrupt HW28
	j OS_Trap_Interrupt_UNDEF   ! (38) Interrupt HW29
	j OS_Trap_Interrupt_UNDEF   ! (39) Interrupt HW30
	j OS_Trap_Interrupt_UNDEF   ! (40) Interrupt HW31

	.align  2
exception_vector_end:
exception_handler:
	.long tlb_exception_handler
	.long error_exception_handler
	.long syscall_handler
ISR_TABLE:
	.long HW0_ISR
	.long HW1_ISR
	.long HW2_ISR
	.long HW3_ISR
	.long HW4_ISR
	.long HW5_ISR
	.long HW6_ISR
	.long HW7_ISR
	.long HW8_ISR
	.long HW9_ISR
	.long HW10_ISR
	.long HW11_ISR
	.long HW12_ISR
	.long HW13_ISR
	.long HW14_ISR
	.long HW15_ISR
	.long HW16_ISR
	.long HW17_ISR
	.long HW18_ISR
	.long HW19_ISR
	.long HW20_ISR
	.long HW21_ISR
	.long HW22_ISR
	.long HW23_ISR
	.long HW24_ISR
	.long HW25_ISR
	.long HW26_ISR
	.long HW27_ISR
	.long HW28_ISR
	.long HW29_ISR
	.long HW30_ISR
	.long HW31_ISR
/*
 * exception handlers
 */

/*----------------------------------------------------------------------
   Since N903 doesn't have MMU, make them share the common error handler.
----------------------------------------------------------------------*/
OS_Trap_TLB_Fill:
/*
	SAVE_ALL
	li  $r0, #0x1
	b   tlb_exception_handler
*/

OS_Trap_PTE_Not_Present:
/*
	SAVE_ALL
	li  $r0, #0x2
	b   tlb_exception_handler
*/

OS_Trap_TLB_Misc:
/*
	SAVE_ALL
	li  $r0, #0x3
	b   tlb_exception_handler
*/

OS_Trap_TLB_VLPT_Miss:
1:  b   1b
	SAVE_ALL
//	li  $r0, #0x4
	b   tlb_exception_handler

OS_Trap_Machine_Error:
1:  b   1b
	SAVE_ALL
	li  $r0, #0x5
	b   error_exception_handler

OS_Trap_Debug_Related:
1:  b   1b
	SAVE_ALL
	li  $r0, #0x6
	b   error_exception_handler

OS_Trap_General_Exception:
1:  b   1b
	SAVE_ALL
	li  $r0, #0x7
	b   error_exception_handler

OS_Trap_Syscall:
	SYSCALL_SAVE_ALL
	bal syscall_handler
	SYSCALL_RESTORE_ALL
	iret

OS_Trap_Interrupt_HW0:
	SAVE_ALL_HW0
	li  $r0, 0x0
	la  $r1, HW0_ISR
	jral $r1
	RESTORE_ALL_HW0
	iret

common_ISR_wrapper:
	SAVE_ALL_HW
	la  $r1, ISR_TABLE
	lw  $r1, [$r1+$r0<<2]
	jral $r1
	RESTORE_ALL_HW
	iret

OS_Trap_Interrupt_UNDEF:
	push $r0
	li  $r0, 0xff
	b   common_ISR_wrapper

OS_Trap_Interrupt_HW1:
	push $r0
	li   $r0, 0x01
	b    common_ISR_wrapper

OS_Trap_Interrupt_HW9:
	push $r0
	li  $r0, 0x09
	b   common_ISR_wrapper

OS_Trap_Interrupt_HW13:
	push $r0
	li  $r0, 0x0D
	b   common_ISR_wrapper

OS_Trap_Interrupt_HW19:
	push $r0
	li  $r0, 0x13
	b   common_ISR_wrapper

OS_Trap_SWI:
	SAVE_ALL
	li  $r0, 0x0f
	la  $r1, SW0_ISR
	jral $r1
	RESTORE_ALL
	iret

##------------------------------------------------------------------------------
## Startup code

#include "config.h"
#ifdef CFG_16MB
#include "ag101_16mb.h"
#else
#ifdef CFG_AG102
#include "ag102.h"
#else
#include "ag101.h"
#endif
#endif

#ifdef CFG_LLINIT
#ifdef CFG_REMAP
#define LED_VALUE (0x66) //Burn
#else
#define LED_VALUE (0x11) //xIp
#endif
#else
#define LED_VALUE (0x00) //lOad
#endif

	.section .text
	.global _start
	.weak _call_exit
	.weak _SDA_BASE_
	.weak _FP_BASE_
	.func _start
	.type _start, @function

	.align 2
_start:
	!************************** Begin of do-not-modify **************************
	! Please don’t modify this code
	! Initialize the registers used by the compiler
#ifndef CONFIG_NO_NDS32_EXT_EX9
	! make sure the instruction before setting ITB
	! will not be optimized with ex9
	.no_ex9_begin           ! disable ex9 generation
#endif

	! Support Relax, Set $gp to _SDA_BASE_
	la  $gp, _SDA_BASE_     ! init GP for small data access

#ifdef CFG_DEBUG
	mfsr $r0, $MSC_CFG

#ifndef CONFIG_NO_NDS32_EXT_EX9
	li  $r1, (1 << 24)      ! EIT
	and $r2, $r0, $r1
1:  beqz $r2, 1b
#endif

#ifdef NDS32_EXT_IFC
	li  $r1, (1 << 19)      ! IFC
	and $r2, $r0, $r1
1:  beqz $r2, 1b
#endif

#endif //#ifdef CFG_DEBUG

#ifndef CONFIG_NO_NDS32_EXT_EX9
	! Initialize the table base of EX9 instruction
	la      $r0, _ITB_BASE_ ! init ITB
	mtusr   $r0, $ITB
	.no_ex9_end
#endif
	!*************************** End of do-not-modify ***************************

	la  $fp, _FP_BASE_      ! init FP
	la  $sp, _stack         ! init SP
#ifndef CFG_SIMU
#ifndef CFG_AG102
	li  $r0, LED_VALUE
	bal set_led
#endif
#endif
#ifdef CFG_LLINIT
	bal _nds32_init_mem
#endif
	bal __init
	bal main
1:  b   1b

#ifndef CFG_SIMU
#ifndef CFG_AG102
	.global set_led
set_led:
	! light LED
	li  $r1, LED_BASE
	sw  $r0, [$r1]
	ret
#endif
#endif

	.size _start, .-_start

	.end
