#include <nds32_intrinsic.h>
#include "config.h"
#ifdef CFG_16MB
#include "ag101_16mb.h"
#else
#ifdef CFG_AG102
#include "ag102.h"
#else
#include "ag101.h"
#endif
#endif

extern int uart_puts(const char *s);

inline void GIE_ENABLE()
{
	__nds32__setgie_en();
	__nds32__dsb();
}

inline void GIE_DISABLE()
{
	__nds32__setgie_dis();
	__nds32__dsb();
}

 /* this function generates a s/w interrupt */
void gen_swi()
{
	unsigned int int_pend;

	int_pend = __nds32__mfsr(NDS32_SR_INT_PEND);
	int_pend |= 0x10000;
	__nds32__mtsr(int_pend, NDS32_SR_INT_PEND);
	__nds32__dsb();
}

/* Set the corresponding IRQ source to be routed to HW0 ~ HW5. */
void setIRQ_ivic(unsigned int vector, int hw)
{
#ifdef CFG_AG102
	int n = vector / 8, bits_shift = (vector & 0x7 ) << 2;
	unsigned int val;

	val = inw(INTC_BASE + IRQPRI_OFFSET(n)) & ~(0xf << bits_shift);
	val |= ((hw & 0xf) << bits_shift);
	outw(INTC_BASE + IRQPRI_OFFSET(n), val);
#else
	/* Since FIQ/IRQ is routed to HW0/HW1 by hardware, it is needless for AG101/AG101P */
#endif
}

void enableIntr(unsigned int mask, int edge, int neg)
{
	if (edge)
		outw(INTC_BASE + IRQTMR_OFFSET, inw(INTC_BASE + IRQTMR_OFFSET) | mask);     //Trigger Mode , edge
	else
		outw(INTC_BASE + IRQTMR_OFFSET, inw(INTC_BASE + IRQTMR_OFFSET) & ~mask);    //Trigger Mode, level

	if (neg)
		outw(INTC_BASE + IRQTLR_OFFSET, inw(INTC_BASE + IRQTLR_OFFSET) | mask);     //Trigger Level, active high
	else
		outw(INTC_BASE + IRQTLR_OFFSET, inw(INTC_BASE + IRQTLR_OFFSET) & ~mask);    //Trigger Level, active low

	outw(INTC_BASE + IRQER_OFFSET, inw(INTC_BASE + IRQER_OFFSET) | mask);           //Enable Register

}

void enableFiqIntr(unsigned int mask, int edge, int neg)
{
	if (edge)
		outw(INTC_BASE + FIQTMR_OFFSET, inw(INTC_BASE + FIQTMR_OFFSET) | mask);     //Trigger Mode , edge
	else
		outw(INTC_BASE + FIQTMR_OFFSET, inw(INTC_BASE + FIQTMR_OFFSET) & ~mask);    //Trigger Mode, level

	if (neg)
		outw(INTC_BASE + FIQTLR_OFFSET, inw(INTC_BASE + FIQTLR_OFFSET) | mask);     //Trigger Level, active high
	else
		outw(INTC_BASE + FIQTLR_OFFSET, inw(INTC_BASE + FIQTLR_OFFSET) & ~mask);    //Trigger Level, active low

	outw(INTC_BASE + FIQER_OFFSET, inw(INTC_BASE + FIQER_OFFSET) | mask);           //Enable Register

}

#if (defined(NDS32_BASELINE_V3) || defined(NDS32_BASELINE_V3M))
unsigned int intr_save_mask(unsigned int num)
{
	/* get current intr enable mask */
	unsigned int old_mask = __nds32__mfsr(NDS32_SR_INT_MASK2);
	unsigned int mask = old_mask & (~(1 << num));
	__nds32__mtsr(mask, NDS32_SR_INT_MASK2);
	__nds32__dsb();
	return old_mask;
}

void intr_restore_mask(unsigned int mask)
{
	GIE_DISABLE();
	/* restore intr enable mask */
	__nds32__mtsr(mask, NDS32_SR_INT_MASK2);
	__nds32__dsb();
}
#endif

#define MHz 1000000
#define TICK_HZ 1
#ifdef CFG_AG102
#define MB_PCLK (60 * MHz)
#else
#ifdef CFG_16MB
#define MB_PCLK (15 * MHz)
#else
#define MB_PCLK (15 * MHz)
#endif
#endif

#ifdef CFG_SIMU
 #define SIMU_FACTOR (8)
 #define SET_LED(x)
#else
 #define SIMU_FACTOR (0)
#ifdef CFG_AG102
 #define SET_LED(x)
#else
 #define SET_LED(x) outw(LED_BASE, x)
#endif
#endif

void Tmr_TickInit()
{
	/* timer1 will interrupt in every 4 secs */
	unsigned int period;
	period = (4 * (MB_PCLK / TICK_HZ)) >> SIMU_FACTOR;
	outw(TM1_LOAD, period);
	outw(TM1_CNTR, period);
	outw(TM1_MTCH1, 0);
	outw(TM_MSK, (TM_STA_TM1M1));
	outw(TM_CR, (TM_CR_TM1ENA | TM_CR_TM1OEN));
}

void intc_reset()
{
	/* HW1(+0x4)Enable Regiter, interrupt disabled */
	outw(INTC_BASE + IRQER_OFFSET, INT_ER_INIT);
	/* HW1(+0x8)Interrupt Clear Register, write 1 to clear */
	outw(INTC_BASE + IRQICR_OFFSET, INT_CLEAR_VALUE);

#ifndef CFG_AG102
	/* HW0(+0x24)Enable Regiter, interrupt disabled */
	outw(INTC_BASE + FIQER_OFFSET, INT_ER_INIT);
	/* HW0(+0x28)Interrupt Clear Register, write 1 to clear */
	outw(INTC_BASE + FIQICR_OFFSET, INT_CLEAR_VALUE);
#endif
}

void initIntr()
{
	/* Init GPIO */
	outw(GPIO_BASE + GPIO_DIR_OFFSET, inw(GPIO_BASE + GPIO_DIR_OFFSET) & ~GPIO_USED_MASK);
	outw(GPIO_BASE + GPIO_IR_OFFSET, inw(GPIO_BASE + GPIO_IR_OFFSET) | GPIO_USED_MASK);
	outw(GPIO_BASE + GPIO_IE_OFFSET, GPIO_USED_MASK);
	/* set the max value to debounce */
	outw(GPIO_BASE + GPIO_BPS_OFFSET, 0xFFFF);
	/* enable debounce */
	outw(GPIO_BASE + GPIO_BE_OFFSET, GPIO_USED_MASK);

	/* Init timer */
	Tmr_TickInit();

	/* Check IVIC version (IVB.IVIC_VER) */
	if (__nds32__mfsr(NDS32_SR_IVB) & (1 << 11)) {
#if (defined(NDS32_BASELINE_V3) || defined(NDS32_BASELINE_V3M))
        	/* enable HW# (timer1, GPIO & SWI) */
        	__nds32__mtsr(IC_TIMER1 | IC_GPIO | IC_SWI, NDS32_SR_INT_MASK2);
		/* Interrupt pending register, write 1 to clear */
		__nds32__mtsr(0xFFFFFFFF, NDS32_SR_INT_PEND2);
#endif
	} else {
		/* Reset interrupt controller */
        	intc_reset();

		/* IRQ source to be routed to HW0/HW1 */
		setIRQ_ivic(IRQ_TIMER1, 1);
		setIRQ_ivic(IRQ_GPIO, 0);

		/* Timer is edge-trigger and falling-edge */
		enableIntr(IC_TIMER1, 1, 0);
		/* GPIO is level-trigger and active-high */
		enableFiqIntr(IC_GPIO, 0, 0);

		/* enable SW0, HW0 and HW1 */
		__nds32__mtsr(0x10003, NDS32_SR_INT_MASK);
	}
}

void initIntr_standby()
{
	/* Init GPIO */
	outw(GPIO_BASE + GPIO_DIR_OFFSET, inw(GPIO_BASE + GPIO_DIR_OFFSET) & ~GPIO_USED_MASK);
	outw(GPIO_BASE + GPIO_IR_OFFSET, inw(GPIO_BASE + GPIO_IR_OFFSET) | GPIO_USED_MASK);
	outw(GPIO_BASE + GPIO_IE_OFFSET, GPIO_USED_MASK);
	/* set the max value to debounce */
	outw(GPIO_BASE + GPIO_BPS_OFFSET, 0xFFFF);
	/* enable debounce */
	outw(GPIO_BASE + GPIO_BE_OFFSET, GPIO_USED_MASK);

	/* Check IVIC version (IVB.IVIC_VER) */
	if (__nds32__mfsr(NDS32_SR_IVB) & (1 << 11)) {
#if (defined(NDS32_BASELINE_V3) || defined(NDS32_BASELINE_V3M))
		/* enable HW (GPIO) */
		__nds32__mtsr(IC_GPIO, NDS32_SR_INT_MASK2);
		/* Interrupt pending register, write 1 to clear */
		__nds32__mtsr(0xFFFFFFFF, NDS32_SR_INT_PEND2);
#endif
	} else {
		/* Reset interrupt controller */
		intc_reset();
		/* GPIO source to be routed to HW0 */
                setIRQ_ivic(IRQ_GPIO, 0);
		/* GPIO is level-trigger and active-high */
		enableFiqIntr(IC_GPIO, 0, 0);
		/* enable HW0 */
		__nds32__mtsr(0x0001, NDS32_SR_INT_MASK);
	}
}

void clear_swi()
{
	unsigned int int_pend;

	int_pend = 0x10;
	__nds32__mtsr(int_pend, NDS32_SR_INT_PEND);
	__nds32__dsb();
}

/* User can use our handler or Default_Handler */
#if 0
void exception_handler(int exception_nr)
{
	SET_LED(exception_nr);
	uart_puts("exception handler\n");
	while (1) ;
}

void tlb_exception_handler(int exception_nr)
{
	SET_LED(exception_nr);
	uart_puts("tlb exception handler\n");
	while (1) ;
}

void error_exception_handler(int exception_nr)
{
	SET_LED(exception_nr);
	uart_puts("error exception handler\n");
	while (1) ;
}
#endif

#if 0
#include <stdio.h>
void syscall_handler(
	unsigned int arg0,
	unsigned int arg1,
	unsigned int arg2,
	unsigned int arg3,
	unsigned int arg4,
	unsigned int arg5)
{
	char buf[0x100];
	unsigned swid = 0x7FFF & (__nds32__mfsr(NDS32_SR_ITYPE) >> 16);

	sprintf(buf, "syscall_0x%08x(0x%08x,0x%08x,0x%08x,0x%08x,0x%08x,0x%08x)\n"
		, swid, arg0, arg1, arg2, arg3, arg4, arg5);
	uart_puts(buf);
}
#endif

/* HW0, HW1 and SW0 interrupts will only occur on IVIC version (IVB.IVIC_VER) = 0. */
void HW0_ISR()
{
	uart_puts("* Enter HW0 ISR with GPIO *\n");
	outw(GPIO_BASE + GPIO_IC_OFFSET, 0xFFFFFFFF);

	/* This service will take 6 secs */
	unsigned int period;
	period = (6 * (MB_PCLK / TICK_HZ)) >> SIMU_FACTOR;
	outw(TM2_LOAD, period);
	outw(TM2_CNTR, 0);
	outw(TM_CR, (inw(TM_CR) | TM_CR_TM2ENA | TM_CR_TM2UP));
	while (1) {
		unsigned int tm2_cntr = inw(TM2_CNTR);
		if (tm2_cntr > period)
			break;
	}
	uart_puts("* End of HW0 ISR it takes 6 secs *\n");
}

void HW1_ISR()
{
	uart_puts("* Enter HW1 ISR, It comes in every 4 secs. *\n");
	/* Disable Timer interrupt*/
	outw(INTC_BASE + IRQER_OFFSET, inw(INTC_BASE + IRQER_OFFSET) & (~IC_TIMER1));

	/* Clear interrupt status */
	outw(INTC_BASE + IRQICR_OFFSET, IC_TIMER1);
	outw(TM_STA, TM_STA_TM1M1);
	uart_puts("* Top-Half of HW1 is done. Enable GIE *\n");
	GIE_ENABLE();
	/* This service will take 2 secs */
	unsigned int period;
	period = (2 * (MB_PCLK / TICK_HZ)) >> SIMU_FACTOR;
	outw(TM3_LOAD, period);
	outw(TM3_CNTR, 0);
	outw(TM_CR, (inw(TM_CR) | TM_CR_TM3ENA | TM_CR_TM3UP));
	while (1) {
		unsigned int tm3_cntr = inw(TM3_CNTR);
		if (tm3_cntr > period)
			break;
	}
	uart_puts("* Bottom-Half of HW1 is done and it takes 2 secs. Enable it self.*\n");

	/* Enable Timer Interrupt*/
	outw(INTC_BASE + IRQER_OFFSET, inw(INTC_BASE + IRQER_OFFSET) | IC_TIMER1);
}

void SW0_ISR()
{
	uart_puts("A software interrupt occurs ...\n");
	clear_swi();
	return;
}

#if (defined(NDS32_BASELINE_V3) || defined(NDS32_BASELINE_V3M))
/* HW9, HW13 and HW19 interrupt will only occur on IVIC version (IVB.IVIC_VER) = 1. */
void HW9_ISR(unsigned int num)
{
	__nds32__mtsr((1 << num), NDS32_SR_INT_PEND2);
	SW0_ISR();
}

void HW13_ISR(unsigned int num)
{
	unsigned int msk = intr_save_mask(num);

	uart_puts("* Enter HW13 ISR with GPIO *\n");
	__nds32__mtsr((1 << num), NDS32_SR_INT_PEND2);
	outw(GPIO_BASE + GPIO_IC_OFFSET, 0xFFFFFFFF);
	GIE_ENABLE();

	/* This service will take 6 secs */
	unsigned int period;
	period = (6 * (MB_PCLK / TICK_HZ)) >> SIMU_FACTOR;
	outw(TM2_LOAD, period);
	outw(TM2_CNTR, 0);
	outw(TM_CR, (inw(TM_CR) | TM_CR_TM2ENA | TM_CR_TM2UP));
	while (1) {
		unsigned int tm2_cntr = inw(TM2_CNTR);
		if (tm2_cntr > period)
			break;
	}
	uart_puts("* End of HW13 ISR it takes 6 secs *\n");
	intr_restore_mask(msk);
}

void HW19_ISR(unsigned int num)
{
	unsigned int msk = intr_save_mask(num);

	uart_puts("* Enter HW19 ISR, It comes in every 4 secs. *\n");
	/* Clear interrupt status */
	__nds32__mtsr((1 << num), NDS32_SR_INT_PEND2);
	outw(TM_STA, TM_STA_TM1M1);
	uart_puts("* Top-Half of HW19 is done. Enable GIE *\n");
	GIE_ENABLE();
	/* This service will take 2 secs */
	unsigned int period;
	period = (2 * (MB_PCLK / TICK_HZ)) >> SIMU_FACTOR;
	outw(TM3_LOAD, period);
	outw(TM3_CNTR, 0);
	outw(TM_CR, (inw(TM_CR) | TM_CR_TM3ENA | TM_CR_TM3UP));
	while (1) {
		unsigned int tm3_cntr = inw(TM3_CNTR);
		if (tm3_cntr > period)
			break;
	}
	uart_puts("* Bottom-Half of HW19 is done and it takes 2 secs. Enable it self.*\n");
	intr_restore_mask(msk);
}
#endif

#if 0
void SYSirq_Disable_Interrupts_Save_Flags(unsigned int *flags)
{
	unsigned int tmp_flags;

	*flags = __nds32__mfsr(NDS32_SR_PSW) & 0x1;
	__nds32__setgie_dis();
	__nds32__dsb();
}

void SYSirq_Enable_Interrupts_Save_Flags(unsigned int *flags)
{
	unsigned int tmp_flags;

	*flags = __nds32__mfsr(NDS32_SR_PSW) & 0x1;
	__nds32__setgie_en();
	__nds32__dsb();
}

void SYSirq_Interrupts_Restore_Flags(unsigned int flags)
{
	unsigned int tmp;

	tmp = __nds32__mfsr(NDS32_SR_PSW);
	tmp = __nds32__bclr(tmp, 0) | flags;
	__nds32__mtsr(tmp, NDS32_SR_PSW);
	__nds32__dsb();
}
#endif
