#include "config.h"
#ifdef CFG_16MB
#include "ag101_16mb.h"
#else
#ifdef CFG_AG102
#include "ag102.h"
#else
#include "ag101.h"
#endif
#endif

#ifdef CFG_LLINIT
void uart_init(void)
{
	/* mode select (UART) */
	outw(UART_BASE + UART_MDR_OFFSET,
	     inw(UART_BASE + UART_MDR_OFFSET) & ~(0x3));
	/* Set DLAB to 1 */
	outw(UART_BASE + UART_LCR_OFFSET,
	     inw(UART_BASE + UART_LCR_OFFSET) | 0x80);
#ifdef CFG_AG102
	/* DLL = 0x18 */
        outw(UART_BASE + 0x00, 0x36);
#else
	/* DLL = 0x18 */
	outw(UART_BASE + 0x00, 0x18);
#endif
	/* DLM = 0 */
	outw(UART_BASE + 0x04, 0x0);
	/* baud 38400 , LCR: Length 8, no parity, 1 stop bit. */
	outw(UART_BASE + UART_LCR_OFFSET, 0x03);
	/* FCR: Enable FIFO, reset TX and RX. */
	outw(UART_BASE + UART_FCR_OFFSET, 0x07);
}
#endif

void bsp_outbyte(unsigned char c)
{
	/* tx fifo full */
	while (inw(UART_BASE + 0x08) & 0x10) ;
	outw(UART_BASE + 0x00, c);
}

#define SERIAL_THR                     	0x00
#define SERIAL_LSR                      0x14
#define SERIAL_LSR_THRE                 0x20

int uart_puts(const char *s)
{
	volatile unsigned *status =
	    (volatile unsigned *)(UART_BASE + SERIAL_LSR);
	int len = 0;
	while (*s) {
		while ((*status & SERIAL_LSR_THRE) == 0) ;

		*(volatile unsigned *)(UART_BASE + SERIAL_THR) = (unsigned)*s;

		if (*s == '\n') {
			while ((*status & SERIAL_LSR_THRE) == 0) ;
			*(volatile unsigned *)(UART_BASE + SERIAL_THR) = '\r';
		}
		s++;
		len++;
	}
	return len;
}
